/*****************************************************************************
 *
 * FILE:	rc4.c
 * DESCRIPTION:	RC4: Crypt library
 * DATE:	Thu, Jun 15 2006
 * UPDATED:	Mon, Jun 19 2006
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2006 졿Kouichi ABE (WALL), All rights reserved.
 * NOTES:	cf. http://www.columbia.edu/~ariel/ssleay/rrc4.html
 * LICENSE:
 *
 *  Copyright (c) 2006 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 *
 * $Id: rc4.c,v 1.2 2007/09/02 19:11:24 kouichi Exp $
 *
 *****************************************************************************/

#include <sys/types.h>
#include "rc4.h"

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static void	swap_byte(unsigned char * a, unsigned char * b);

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/
void
rc4_set_key(RC4_KEY * key,
	    const unsigned char * key_data_ptr, size_t key_data_len)
{
  unsigned char		index1;
  unsigned char		index2;
  unsigned char *	state;
  short			counter;

  state = &key->state[0];
  for (counter = 0; counter < RC4_KEY_STATE_SIZE; counter++) {
    state[counter] = counter;
  }
  key->x = 0;
  key->y = 0;
  index1 = 0;
  index2 = 0;
  for (counter = 0; counter < RC4_KEY_STATE_SIZE; counter++) {
    index2 = (key_data_ptr[index1] + state[counter] + index2)
	   % RC4_KEY_STATE_SIZE;
    swap_byte(&state[counter], &state[index2]);

    index1 = (index1 + 1) % key_data_len;
  }
}

void
rc4(RC4_KEY * key, unsigned char * buffer_ptr, size_t buffer_len)
{
  unsigned char		x;
  unsigned char		y;
  unsigned char *	state;
  unsigned char		xorIndex;
  short			counter;

  x = key->x;
  y = key->y;

  state = &key->state[0];
  for (counter = 0; counter < buffer_len; counter ++) {
    x = (x + 1) % RC4_KEY_STATE_SIZE;
    y = (state[x] + y) % RC4_KEY_STATE_SIZE;
    swap_byte(&state[x], &state[y]);

    xorIndex = (state[x] + state[y]) % RC4_KEY_STATE_SIZE;

    buffer_ptr[counter] ^= state[xorIndex];
  }
  key->x = x;
  key->y = y;
}

static void
swap_byte(unsigned char * a, unsigned char * b)
{
  unsigned char	swapByte;

  swapByte = *a;
  *a = *b;
  *b = swapByte;
}
